#!/usr/local/bin/python
from flask import Flask
from flask import request
from flask import Response
import traceback, sys, datetime
from Jupytils.DBUtils import DBUtils

app = Flask(__name__)
#----------------------------------------------------------------------
class FlaskSQLWS:
    def __init__(self):
        pass;
#----------------------------------------------------------------------
    @app.route('/', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
    def ws():
        defr = '''Data Service: Version 1.0'''
        return Response(defr, mimetype='text');
#----------------------------------------------------------------------
    @app.route('/echo', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
    def echo():
        tags = [t+"="+request.args[t] for t in request.args ]
        reqs =  str(request.json)
        t = str(tags)  + "\n" + reqs + "\n"
        return t
#--------------------------------------------------------------------------
    @app.route('/q/<qid>/', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
    @app.route('/q/<type>/<qid>/', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
    def q(type = 'html', qid=''):
        if ( qid in db.QCACHE ):
            q1 = db.QCACHE[qid]
        else:
            q1 = qid;

        try: 
            q2, tags = DBUtils.prepare(q1, request.args);
            if (type not in ['html', 'text', 'json', 'csv'] ):
                type = 'html';
            print(q2, " ==>", type)
            df = db.execQ(q2)

            if (type == 'html'):
                r = q2 + "<br/><br/>"
                r = r + df.to_html();
                return Response(r, mimetype='text/html');
            elif (type == 'text' or type == 'csv' ):
                r = df.to_csv(index=False);
                return Response(r, mimetype='text');
            else:
                r = df.to_json();
                return Response(r, mimetype='text/json');
        except Exception as E:
            #E = sys.exc_info()[0]
            traceback.print_exc(file=sys.stdout)
            r = "Exception: " + qid + ":" + type + "\n\n" + q1 + "\n\n" + q2 + "\n\n"+ str(E) +"\n"+df;
            return Response(r, mimetype="text");
#--------------------------------------------------------------------------
    @app.route('/dump', methods=['POST', 'GET', 'OPTIONS', 'HEAD'])
    def dump():
        ret = db.SQL
        return  Response(ret, mimetype='text')
#---------------------------------------------------------------------
    def start(self, port=8500, sqlText=None, conn=None):
        global db;
        if (sqlText is not None):
            db = DBUtils(conn = conn);
            db.load(file=sqlText);

        print( "Starting at port: ", port)
        app.run(debug=True, host="0.0.0.0", port=port)
#--------------------------------------------------------------------------
if __name__ == '__main__':
    import os.path;
    if not os.path.exists('SQL.txt'):
        with open("SQL.txt", "w") as f:
            f.write("#---------\nex1\n\nSELECT * from test Limit 10\n#---------")

    s = "SQL.txt";
    conn = 'postgresql://postgres:postgres@localhost/SCHASDB'
    f = FlaskSQLWS()
    f.start(sqlText=s, conn=conn)


from __future__ import absolute_import

"""Functions to quickly display charts in a Notebook.
"""

import string
import random
import json
import copy
import pandas as pd

from IPython.core import getipython
from IPython.core.display import display, HTML
import numpy as np;

# Note that Highstock includes all Highcharts features.
HIGHCHARTS_SCRIPTS = """
<script src="//code.highcharts.com/stock/highstock.js"></script>
<script src="//code.highcharts.com/highcharts-more.js"></script>
<script src="//code.highcharts.com/modules/exporting.js"></script>
<script src="https://code.highcharts.com/highcharts.js"></script>
<script src="https://code.highcharts.com/modules/histogram-bellcurve.js"></script>
"""


def load_highcharts():
    return display(HTML(HIGHCHARTS_SCRIPTS))

# Automatically insert the script tag into your Notebook.
# Call when you import this module.
if 'IPKernelApp' in getipython.get_ipython().config:
    load_highcharts()



def PlotHCts(df, x, cols=[], div=None, title='', subtitle='', yTitle='',xTitle='', num=1000000,
          onClick='function(){g=this;console.log(g.index, g.y, g.x)}',
            animation='true', ctype=None):
    TS='''
<script>
Highcharts.chart('CHART_DIV', {
    chart: { type: 'line' ,  zoomType: 'x' },
    title: { text: 'CHART_TITLE' , zoomType:'xy' },
    subtitle: { text: 'CHART_SUB_TITLE' },
    xAxis: {type: 'datetime'
    },
    yAxis: {
        title: { text: 'CHART_Y_AXIS_TITLE'}
    },
    plotOptions: {
        scatter: {
            marker: { radius: 3},
            states: {
                hover: {
                    enabled: true,
                    lineColor: 'rgb(100,100,100)'
                }
            }
        },
        line: {
            animation: CHART_ANIMATION,
            dataLabels: {  enabled: false  },
            enableMouseTracking: true,
            lineWidth: 0.1,
            marker: { radius: 3} 
            },
            
            series: {
                point: {
                    events: {
                        click: CLICK_FUNCTION
                    }
                }
            }
        },
        DATA

});
</script>
'''
    ts ="";
    if ( div is None ):
        div = 'chart_' + str(np.random.randint(1000000))
        ts = '<div id="{}" style="height:200px"></div>\n'.format(div)
        
    if (type(x) == str):
        xa = df[x]
    elif (type(x) == int):
        xa = df[df.columns[x]]
    else: #(type(x) == pd.core.series.Series)
        xa = x
        
    if (len(xa) <=0 ): return;
    if (xa.astype(int)[0] > 1000000000000000000):
        dt=(xa.astype(int)/1000000)[0:num]
    elif (xa.astype(int)[0] > 1000000000000000):
        dt=(xa.astype(int)/1000)[0:num]
    else:
        dt = xa;
        
    dt = [int(c) for c in dt]
    dd=[]
    
    
    for c in cols:
        if ( type(c) == str):
            d=list(df[c].values[0:num])
            cn=c
        elif( type(c) == int ):
            cn=df.columns[c]
            d=list(df[cn].values[0:num])
        else:
            cn="List"
            d=c;
            
        dat=list(zip(dt, d))
        dd.append({'name': cn, 'data': dat })

    s = 'series: ' + pd.io.json.dumps(dd) +""
    ts= ts + TS.replace('DATA',s)
    ts=ts.replace('CHART_DIV', div)
    ts=ts.replace('CHART_TITLE', title)
    ts=ts.replace('CHART_SUB_TITLE', subtitle)
    ts=ts.replace('CHART_Y_AXIS_TITLE', yTitle)
    ts=ts.replace('CLICK_FUNCTION', onClick)
    ts=ts.replace('CHART_ANIMATION', animation)

    if (ctype != None):
        ts=ts.replace("type: 'line'","type: '{}'".format(ctype));
        
    display(HTML(ts))    
    return ts;

#ts=PlotHCts(df, x=df.sdttm, cols='so2_max precipitation_max'.split(), div='chart1',num=10)
#ts=plotTSHC(df, x=df.sdttm, cols=[23,21,df.temperature_diff, random.random(1999)], title="PEF Values", div='chart1',num=100000, onClick=onClick)
